@@ -86,7 +86,7 @@ def do_movement(options, data: synth_format.DataContainer, filter_types: list =
     if options.rotate:
         _movement_helper(data, movement.rotate, movement.rotate_relative, movement.rotate_around, options.relative, options.pivot, options.rotate, types=filter_types)
     if options.offset:
-        data.apply_for_all(movement.offset, options.offset, types=filter_types)
+        _movement_helper(data, movement.offset, movement.offset_relative, movement.offset, options.relative, options.pivot, options.offset, types=filter_types)
     if options.outset:
         _movement_helper(data, movement.outset, movement.outset_relative, movement.outset_from, options.relative, options.pivot, options.outset, types=filter_types)
 
@@ -124,23 +124,29 @@ def get_parser():
     preproc_group.add_argument("-w", "--change-walls", metavar="NEW_WALL_TYPE", nargs="+", choices=synth_format.WALL_TYPES, help=f"Change the type of walls. Specify multiple to loop over them.")
 
     rail_pattern_group = parser.add_argument_group("rail patterns")
-    rail_pattern_group.add_argument("--interpolate", type=_parse_number, metavar="INTERVAL", help="Subdivide rail into segments of this length in beats, interpolating linearly. Supports fractions. When used with --spiral or --spikes, this is the distance between each nodes")
-    rail_pattern_group.add_argument("--shorten-rails", type=_parse_number, metavar="DISTANCE", help="Cut some distance from every rail, interpolating linearly. Supports fractions. When negative, cuts from the start instead of the end")
+    interp_group = rail_pattern_group.add_mutually_exclusive_group()
+    interp_group.add_argument("--interpolate", type=_parse_number, metavar="INTERVAL", help="Subdivide rail into segments of this length in beats, interpolating using splines (similar to the game). Supports fractions. When used with --spiral or --spikes, this is the distance between each nodes")
+    interp_group.add_argument("--interpolate-linear", type=_parse_number, metavar="INTERVAL", help="Subdivide rail into segments of this length in beats, interpolating linearly. Supports fractions. When used with --spiral or --spikes, this is the distance between each nodes")
+    rail_pattern_group.add_argument("--shorten-rails", type=_parse_number, metavar="DISTANCE", help="Cut some distance from every rail. Supports fractions. When negative, cuts from the start instead of the end")
     rail_pattern_group.add_argument("--start-angle", type=_parse_number, default=0.0, metavar="DEGREES", help="Angle of the first node of the spiral in degrees. Default: 0/right")
     rail_pattern_group.add_argument("--radius", type=_parse_number, default=1.0, help="Radius of spiral or length of spikes")
-    rail_pattern_group.add_argument("--spiral", type=_parse_number, metavar="NODES_PER_ROT", help="Generate counterclockwise spiral around rails with this number of nodes per full rotation. Supports fractions. 2=zigzag, negative=clockwise")
-    rail_pattern_group.add_argument("--spikes", type=_parse_number, metavar="NODES_PER_ROT", help="Generate spikes from rail, either spiraling (see --spiral) or random (when set to 0)")
+    rail_op_group = rail_pattern_group.add_mutually_exclusive_group()
+    rail_op_group.add_argument("--spiral", type=_parse_number, metavar="NODES_PER_ROT", help="Generate counterclockwise spiral around rails with this number of nodes per full rotation. Supports fractions. 2=zigzag, negative=clockwise")
+    rail_op_group.add_argument("--spikes", type=_parse_number, metavar="NODES_PER_ROT", help="Generate spikes from rail, either spiraling (see --spiral) or random (when set to 0)")
     rail_pattern_group.add_argument("--spike-width", type=_parse_number, default=1/32, help="Width of spike 'base' in beats. Supports fractions. Should not be lower than 1/32 (the default) and should be lower than chosen interpolation interval")
 
     movement_group = parser.add_argument_group("movement", description="Operation order is always: scale, rotate, offset, outset")
-    movement_group.add_argument("-p", "--pivot", type=_parse_position, help="Pivot for outset, scale and rotate as x,y,t")
-    movement_group.add_argument("--relative", action="store_true", help="Use first node of rails as pivot for scale/rotate")
+    pivot_group = movement_group.add_mutually_exclusive_group()
+    pivot_group.add_argument("-p", "--pivot", type=_parse_position, help="Pivot for outset, scale and rotate as x,y,t")
+    pivot_group.add_argument("--note-pivot", choices=synth_format.NOTE_TYPES, help="Use position of first matching note as pivot (determined before any operations)")
+    pivot_group.add_argument("--relative", action="store_true", help="Use first node of rails as pivot for outset, scale and rotate, and for walls offset because relative based on rotation")
     movement_group.add_argument("-s", "--scale", type=_parse_position, help="Scale positions by x,y,t. Use negative values to mirror across axis. Does NOT change the size of walls. Time-Scale of 2 means twice as long, not twice as fast.")
     movement_group.add_argument("-r", "--rotate", type=_parse_number, metavar="DEGREES", help="Rotate counterclockwise by this many degrees (negative for clockwise)")
     movement_group.add_argument("-o", "--offset", type=_parse_position, help="Move/Translate by x,y,t")
-    movement_group.add_argument("--outset", type=_parse_number, metavar="DISTANCE", help="Move outwards")
-    movement_group.add_argument("--offset-along", choices=synth_format.NOTE_TYPES, help="Offset objects to follow notes and rails of the specified color")
-    movement_group.add_argument("--rotate-with", choices=synth_format.NOTE_TYPES, help="Rotate and outset the objects to follow notes and rails of the specified color")
+    movement_group.add_argument("--outset", type=_parse_number, metavar="DISTANCE", help="Move outwards (negative for inwards, can move 'across' pivot)")
+    rail_stack_group = movement_group.add_mutually_exclusive_group()
+    rail_stack_group.add_argument("--offset-along", choices=synth_format.NOTE_TYPES, help="Offset objects to follow notes and rails of the specified color")
+    rail_stack_group.add_argument("--rotate-with", choices=synth_format.NOTE_TYPES, help="Rotate and outset the objects to follow notes and rails of the specified color")
     movement_group.add_argument("--offset-random", type=_parse_xy_range, metavar="[MIN_X:]MAX_X,[MIN_Y:]MAX_Y", help="Offset by a random amount in the X and Y axis. When no MIN is given, uses negative MAX.")
 
     movement_group.add_argument("-c", "--stack-count", type=int, help="Instead of moving, create copies. Must have time offset set.")
@@ -157,6 +163,12 @@ def abort(reason: str):
     exit(1)
 
 def main(options):
+    try:
+        data = synth_format.import_clipboard(options.use_original)
+    except (JSONDecodeError, KeyError) as err:
+        abort(f"Could not decode clipboard, did you copy somethinge else?\n\t{err!r}")
+
+    # argument post-parsing
     if any(name in options.filter_types for name in _filter_groups):
         out_filters = []
         for inp in options.filter_types:
@@ -173,10 +185,11 @@ def main(options):
             t for t in synth_format.ALL_TYPES
             if t not in options.filter_types
         )
-    try:
-        data = synth_format.import_clipboard(options.use_original)
-    except (JSONDecodeError, KeyError) as err:
-        abort(f"Could not decode clipboard, did you copy somethinge else?\n\t{err!r}")
+    if options.note_pivot:
+        notes = getattr(data, options.note_pivot)
+        if not notes:
+            abort(f"Could not find any {options.note_pivot} notes")
+        options.pivot = notes[sorted(notes)[0]][:3]
 
     # preprocessing
     if options.bpm:
@@ -240,7 +253,9 @@ def main(options):
 
     # rail patterns
     if options.interpolate:
-        data.apply_for_notes(rails.interpolate_nodes_linear, options.interpolate, types=filter_types)
+        data.apply_for_notes(rails.interpolate_nodes, "spline", options.interpolate, types=filter_types)
+    elif options.interpolate_linear:
+        data.apply_for_notes(rails.interpolate_nodes, "linear", options.interpolate_linear, types=filter_types)
     if options.shorten_rails:
         data.apply_for_notes(rails.shorten_rail, options.shorten_rails, types=filter_types)
     if options.spiral:
